# Janos version 1.2.1
# JanosModelLexer.py
# by Antoine Rauzy
# Copyrights (c) 2018-2022 AltaRica Association
#
# This file is part of Janos, a pedagogical stochastic simulator.
# Janos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Janos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Janos.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Lexer

# 1) Imported modules
# -------------------

from S2MLCore import *

from JanosType import *
from JanosToken import *

# 2) Lexer
# --------

# 2.1) Creator
# ------------

class JanosModelLexer(S2MLModelLexer):

  def __init__(self, engine):
    S2MLModelLexer.__init__(self, engine)

# 2.2) Management of actions
# --------------------------

  def RegisterActions(self):
    S2MLModelLexer.RegisterActions(self)
    self.RegisterAction(":", JanosModelLexer.ActionColon)

  def ActionColon(self, c):
    c = self.GetChar()
    if c=="=":
      c = self.GetChar()
      if c==":":
        self.tokenString = ":=:"
        self.NewToken(JanosToken.TOKEN_DOUBLE_ASSIGN)
      else:
        self.PutCharBack(c)
        self.tokenString = ":="
        self.NewToken(JanosToken.TOKEN_ASSIGN)
    else:
      self.PutCharBack(c)
      self.tokenString = ":"
      self.NewToken(JanosToken.TOKEN_COLON)

# 2.3) Management of keywords
# ---------------------------

  def RegisterTokens(self):
    S2MLModelLexer.RegisterTokens(self)

  def RegisterKeywords(self):
    S2MLModelLexer.RegisterKeywords(self)
    self.RegisterKeyword("assertion", JanosToken.KEYWORD_ASSERTION)
    self.RegisterKeyword("observer", JanosToken.KEYWORD_OBSERVER)

