# S2MLCore version 1.1.1
# S2MLTerm.py
# by Antoine Rauzy
# Copyrights (c) 2018-2022 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Terms

# 1) Imported modules
# -------------------

from S2MLPath import *

# 2) Terms
# --------

class S2MLTerm:
  def __init__(self, type):
    self._type = type
    self._functor = None
    self._referencePath = None
    self._referredElement = None
    self._value = None
    self._children = []
    self._attributes = dict()

  def GetType(self):
    return self._type

  def GetFunctor(self):
    return self._functor
  def SetFunctor(self, functor):
    self._functor = functor

  def GetReferencePath(self):
    return self._referencePath
  def SetReferencePath(self, path):
    self._referencePath = path

  def GetReferredElement(self):
    return self._referredElement
  def SetReferredElement(self, element):
    self._referredElement = element

  def GetValue(self):
    return self._value
  def SetValue(self, value):
    self._value = value

  def GetNumberOfChildren(self):
    return len(self._children)
  def GetChild(self, i):
    if i<0 or i>=len(self._children):
      return None
    return self._children[i]
  def GetChildren(self):
    return self._children
  def AppendChild(self, child):
    self._children.append(child)
  def AppendChildren(self, children):
    self._children.extend(children)

  def GetAttribute(self, name, defaultValue = None):
    return self._attributes.get(name, defaultValue)
  def SetAttribute(self, name, value):
    self._attributes[name] = value
  def ResetAttribute(self, name):
    self._attributes.pop(name, None)
  def ResetAttributes(self):
    self._attributes.clear()
