# S2MLCore version 1.1.1
# S2MLManagedValue.py
# by Antoine Rauzy
# Copyrights (c) 2018-2022 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creators
# 3) Access methods

# 1) Imported modules
# -------------------

import sys

# 2) Creators
# -----------

class S2MLManagedValue:
  NONE = 0
  BOOLEAN = 1
  INTEGER = 2
  REAL = 3
  IDENTIFIER = 4
  SYMBOL = 5
  STRING = 6
  LIST = 7

  def __init__(self, type, value):
    self._type = type
    self._value = value

  @staticmethod
  def NewNone():
    return S2MLManagedValue(S2MLManagedValue.NONE, None)

  @staticmethod
  def NewBoolean(value):
    return S2MLManagedValue(S2MLManagedValue.BOOLEAN, value)

  @staticmethod
  def NewInteger(value):
    return S2MLManagedValue(S2MLManagedValue.INTEGER, value)

  @staticmethod
  def NewReal(value):
    return S2MLManagedValue(S2MLManagedValue.REAL, value)

  @staticmethod
  def NewIdentifier(value):
    return S2MLManagedValue(S2MLManagedValue.IDENTIFIER, value)

  @staticmethod
  def NewSymbol(value):
    return S2MLManagedValue(S2MLManagedValue.SYMBOL, value)

  @staticmethod
  def NewString(value):
    return S2MLManagedValue(S2MLManagedValue.STRING, value)

  @staticmethod
  def NewList(value = []):
    return S2MLManagedValue(S2MLManagedValue.LIST, value)

# 3) Access methods
# -----------------

  def GetType(self):
    return self._type

  def IsNone(self):
    return self._type==S2MLManagedValue.NONE

  def IsBoolean(self):
    return self._type==S2MLManagedValue.BOOLEAN

  def IsFalse(self):
    return self._type==S2MLManagedValue.BOOLEAN and not self._value

  def IsTrue(self):
    return self._type==S2MLManagedValue.BOOLEAN and self._value

  def IsInteger(self):
    return self._type==S2MLManagedValue.INTEGER

  def IsReal(self):
    return self._type==S2MLManagedValue.REAL

  def IsNumber(self):
    return self.IsInteger() or self.IsReal()

  def IsIdentifier(self):
    return self._type==S2MLManagedValue.IDENTIFIER

  def IsSymbol(self):
    return self._type==S2MLManagedValue.SYMBOL

  def IsGivenSymbol(self, symbol):
    return self._type==S2MLManagedValue.SYMBOL and self._value==symbol

  def IsString(self):
    return self._type==S2MLManagedValue.STRING

  def IsList(self):
    return self._type==S2MLManagedValue.LIST

  def IsListOfReals(self):
    if not self.IsList():
      return False
    for value in self.GetValueOfList():
     if not value.IsNumber():
       return False
    return True

  def GetValueOfBoolean(self):
    if not self.IsBoolean():
      return None
    return self._value

  def GetValueOfInteger(self):
    if not self.IsInteger():
      return None
    return self._value

  def GetValueOfReal(self):
    if not self.IsNumber():
      return None
    return self._value

  def GetValueOfIdentifier(self):
    if not self.IsIdentifier():
      return None
    return self._value

  def GetValueOfSymbol(self):
    if not self.IsSymbol():
      return None
    return self._value

  def GetValueOfString(self):
    if not self.IsString():
      return None
    return self._value

  def GetValueOfList(self):
    if not self.IsList():
      return None
    return self._value

  def GetValueOfListOfReals(self):
    if not self.IsListOfReals():
      return None
    reals = []
    for managedValue in self._value:
      real = managedValue.GetValueOfReal()
      reals.append(real)
    return reals

  def AppendValue(self, value):
    if not self.IsList():
      return
    self._value.append(value)
